<?php
/**
 * $Horde: odin/data.php,v 1.25 2005/01/03 14:35:23 jan Exp $
 *
 * Copyright 2001-2005 Jan Schneider <jan@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ODIN_BASE', dirname(__FILE__));
require_once ODIN_BASE . '/lib/base.php';
require_once ODIN_BASE . '/lib/Source.php';
require_once 'Horde/Data.php';
require ODIN_BASE . '/config/attributes.php';

if (!$conf['menu']['import_export']) {
    header('Location: ' . Horde::applicationUrl('index.php', true));
    exit;
}

$error = false;
$actionID = Util::getFormData('actionID');

if (Util::getFormData('csvresults')) {
    // Fetch the current search results
    if (empty($_SESSION['odin_search_results'])) {
        $notification->push(_("No search results are currently available"), 'horde.warning');
        header('Location: ' . Horde::applicationUrl('query.php', true));
        exit;
    }

    require_once ODIN_BASE . '/lib/List.php';
    require_once ODIN_BASE . '/lib/Object.php';
    $rows = Odin_List::unserialize($_SESSION['odin_search_results']);

    $data = array();
    while ($ob = $rows->next()) {
        $tmp = $ob->getAttributes();
        $row = array();
        foreach ($tmp as $key => $attribute) {
            if (substr($key, 0, 2) != '__') {
                $row[$key] = $attribute;
            }
        }
        $data[] = $row;
    }

    $csv = &Horde_Data::singleton('csv');
    $csv->exportFile('query.csv', $data, true);
    exit;
} elseif ($actionID == 'import') {
    $importID = Util::getFormData('importID');
    $dates = array();
    if ($importID == IMPORT_MAPPED) {
        require_once 'Horde/SessionObjects.php';
        $cache = &Horde_SessionObjects::singleton();
        $cacheID = Util::getFormData('cacheID');

        if (empty($cacheID) ||
            !(list($importData, $dest) = $cache->query($cacheID))) {
            $notification->push(_("The data got lost."), 'horde.error');
            $error = true;
        } else {
            $import_type = Util::getFormData('import_type');
            $dataKeys = Util::getFormData('dataKeys', '');
            $appKeys = Util::getFormData('appKeys', '');
            if (empty($dataKeys) || empty($appKeys)) {
                $notification->push(_("You didn't map any fields from the imported file to the corresponding fields in Odin."), 'horde.error');
                $error = true;
            } else {
                $dataKeys = explode("\t", $dataKeys);
                $appKeys = explode("\t", $appKeys);
                $map = array();
                foreach ($appKeys as $key => $app) {
                    $map[$app] = $dataKeys[$key];
                    if ($attributes[$app]['type'] == 'date' ||
                        $attributes[$app]['type'] == 'time' ||
                        $attributes[$app]['type'] == 'datetime') {
                        $dates[$dataKeys[$key]]['type'] = $attributes[$app]['type'];
                        for ($i = 0; $i < min(count($importData), 10); $i++) {
                            if ($importData[$i][$dataKeys[$key]] != '') {
                                $dates[$dataKeys[$key]]['values'][] = $importData[$i][$dataKeys[$key]];
                            }
                        }
                    }
                }

                if (count($dates) > 0) {
                    $cacheID = $cache->storeOid(array($importData, $dest, $map));
                }
            }
        }
    } elseif ($importID == IMPORT_DATETIME) {
        require_once 'Horde/SessionObjects.php';
        $cache = &Horde_SessionObjects::singleton();
        $cacheID = Util::getFormData('cacheID');

        if (empty($cacheID) || !($tmp = $cache->query($cacheID))) {
            $notification->push(_("The data was lost"), 'horde.error');
            $error = true;
        } else {
            $importData = $tmp[0];
            $dest = $tmp[1];
            $map = $tmp[2];
            $delimiter = Util::getFormData('delimiter');
            $format = Util::getFormData('format');
        }
    }

    if ($importID == IMPORT_MAPPED || $importID == IMPORT_DATETIME) {
        if (!$error && count($dates) == 0) {
            $driver = &Odin_Source::singleton($dest, $cfgSources[$dest]);
            if (is_a($driver, 'PEAR_Error')) {
                Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
                $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
                $error = true;
            } else {
                foreach ($importData as $row) {
                    $hash = array();
                    reset($cfgSources[$dest]['map']);
                    while (list($key, ) = each($cfgSources[$dest]['map'])) {
                        if (isset($map[$key]) && !empty($row[$map[$key]])) {
                            if ($attributes[$key]['type'] == 'date' ||
                                $attributes[$key]['type'] == 'time' ||
                                $attributes[$key]['type'] == 'datetime') {
                                $dt_arr = Horde_Data::mapDate($row[$map[$key]], $attributes[$key]['type'], $delimiter[$map[$key]], $format[$map[$key]]);
                            }
                            if ($attributes[$key]['type'] == 'date' &&
                                $dt_arr['mday'] != 0 &&
                                $dt_arr['month'] != 0 &&
                                $dt_arr['year'] != 0) {
                                $hash[$key] = mktime(0, 0, 0, $dt_arr['month'], $dt_arr['mday'], $dt_arr['year']);
                            } elseif ($attributes[$key]['type'] == 'time') {
                                $hash[$key] = mktime($dt_arr['hour'], $dt_arr['min'], $dt_arr['sec'], 0, 0, 0);
                            } elseif ($attributes[$key]['type'] == 'datetime') {
                                $hash[$key] = mktime($dt_arr['hour'], $dt_arr['min'], $dt_arr['sec'], $dt_arr['month'], $dt_arr['mday'], $dt_arr['year']);
                            } else {
                                $hash[$key] = $row[$map[$key]];
                            }
                        }
                    }
                    if (!$driver->addObject($hash)) {
                        $error = true;
                    }
                }
            }

            if ($error) {
                $notification->push(_("There was an error importing the data."), 'horde.error');
            } else {
                $import_type = Util::getFormData('import_type');
                switch ($import_type) {
                case IMPORT_CSV:
                    $type = 'CSV';
                    break;

                case IMPORT_TSV:
                    $type = 'TSV';
                    break;

                case IMPORT_OUTLOOK:
                    $type = 'Outlook';
                    break;
                }
                $notification->push(sprintf(_("%s file successfully imported"), $type), 'horde.success');
            }
        }
    } else {
        if (!isset($_FILES['import_file']['size']) ||
            !isset($_FILES['import_file']['tmp_name'])) {
            $notification->push(_("There was a problem with the file upload. The file may have been larger than the maximum allowed size."), 'horde.error');
            $error = true;
        } elseif (!is_uploaded_file($_FILES['import_file']['tmp_name'])) {
            $notification->push(_("Illegal or malformed form data."), 'horde.error');
            $error = true;
        } elseif ($_FILES['import_file']['size'] > 0) {
            _fileCleanup($_FILES['import_file']['tmp_name']);

            $header = null;
            switch ($importID) {

            case IMPORT_PROJECTS:
                $project = &Horde_Data::singleton('csv');
                if (!isset($header)) {
                    $header = Util::getFormData('header');
                }
                $importData = $project->importFile($_FILES['import_file']['tmp_name']);
                $dest = Util::getFormData('dest');
                $driver = &Odin_Source::singleton($dest, $cfgSources[$dest]);
                if (!isset($importData) || !is_array($importData)) {
                    $notification->push(_("There was an error importing the uploaded file"), 'horde.error');
                    $error = true;
                } elseif (is_a($driver, 'PEAR_Error')) {
                    Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
                    $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
                    $error = true;
                } else {
                    require_once 'Horde/SessionObjects.php';
                    $cache = &Horde_SessionObjects::singleton();
                    $cacheID = $cache->storeOid(array($importData, $dest));
                    if (empty($cacheID) || !(list($importData, $dest) = $cache->query($cacheID))) {
                        $notification->push(_("The data got lost."), 'horde.error');
                        $error = true;
                    } else {
                        $import_type = Util::getFormData('import_type');
                        $appKeys = array('number', 'title', 'type', 'client', 'city', 'county', 'section',
                                         'township', 'range', 'workstartdate', 'codes', 'clienttype', 'manager');
                        $dataKeys = array(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13);
                        $map = array();
                        foreach ($appKeys as $key => $app) {
                            $map[$app] = $dataKeys[$key];
                        }
                        if ($header) {
                            array_shift($importData);
                        }
                        foreach ($importData as $row) {
                            $hash = array();
                            reset($cfgSources[$dest]['map']);
                            while (list($key, ) = each($cfgSources[$dest]['map'])) {
                                if (isset($map[$key]) && !empty($row[$map[$key]])) {
                                    if ($GLOBALS['attributes'][$key]['type'] == 'date') {
                                        $hash[$key] = strtotime($row[$map[$key]]);
                                    } else {
                                        $hash[$key] = $row[$map[$key]];
                                    }
                                }
                            }
                            if (!$driver->addObject($hash)) {
                                $error = true;
                            }
                        }
                    }
                }

                if ($error) {
                    $notification->push(_("There was an error importing the data."), 'horde.error');
                } else {
                    $notification->push(_("Projects database sucessfully imported"), 'horde.success');
                }

                break;
            }
        } else {
            $notification->push(_("The file contained no data."), 'horde.error');
            $error = true;
        }
    }
} elseif ($actionID == 'export') {
    $exportID = Util::getFormData('exportID');

    switch ($exportID) {
    case EXPORT_CSV:
        $source = Util::getFormData('source');
        if (!isset($source) && isset($cfgSources) && is_array($cfgSources) && count($cfgSources) > 0) {
            reset($cfgSources);
            $source = key($cfgSources);
        }

        $driver = &Odin_Source::singleton($source, $cfgSources[$source]);
        if (is_a($driver, 'PEAR_Error')) {
            Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
            $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
            $error = true;
        } else {
            if (!is_a($addresses = $driver->search(array()), 'Odin_List')) {
                $notification->push(sprintf(_("Failed to search the data source: %s"), $driver->error()), 'horde.error');
                $error = true;
            } else {
                $data = array();
                while ($ob = $addresses->next()) {
                    $tmp = $ob->getAttributes();
                    $row = array();
                    foreach ($tmp as $key => $attribute) {
                        if (substr($key, 0, 2) != '__') {
                            $row[$key] = $attribute;
                        }
                    }
                    $data[] = $row;
                }

                $csv = &Horde_Data::singleton('csv');
                $csv->exportFile('odin.csv', $data, true);
                exit;
            }
        }
        break;

    case EXPORT_TSV:
        $source = Util::getFormData('source');
        if (!isset($source) && isset($cfgSources) && is_array($cfgSources) && count($cfgSources) > 0) {
            reset($cfgSources);
            $source = key($cfgSources);
        }

        $driver = &Odin_Source::singleton($source, $cfgSources[$source]);
        if (is_a($driver, 'PEAR_Error')) {
            Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
            $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
            $error = true;
        } else {
            if (!is_a($addresses = $driver->search(array()), 'Odin_List')) {
                $notification->push(sprintf(_("Failed to search the data source: %s"), $driver->error()), 'horde.error');
                $error = true;
            } else {
                $data = array();
                while ($ob = $addresses->next()) {
                    $tmp = $ob->getAttributes();
                    $row = array();
                    foreach ($tmp as $key => $attribute) {
                        if (substr($key, 0, 2) != '__') {
                            $row[$key] = $attribute;
                        }
                    }
                    $data[] = $row;
                }

                $tsv = &Horde_Data::singleton('tsv');
                $tsv->exportFile('odin.tsv', $data, true);
                exit;
            }
        }
        break;
    }
}

$title = _("Import/Export Databases");
require ODIN_TEMPLATES . '/common-header.inc';
require ODIN_TEMPLATES . '/menu.inc';

if (is_null($actionID) || $error) {
    /* Build the data source select widget. */
    $source_options = '';
    foreach ($cfgSources as $key => $entry) {
        if (!empty($entry['export'])) {
            $source_options .= "<option value=\"$key\">";
            $source_options .= $entry['title'] . "</option>\n";
        }
    }

    /* Build the data source destination select widget. */
    $dest_options = '';
    $hasWriteable = false;
    foreach ($cfgSources as $key => $entry) {
        if (empty($entry['readonly']) || (isset($entry['admin']) && in_array(Auth::getAuth(), $entry['admin']))) {
            $dest_options .= "<option value=\"$key\">";
            $dest_options .= $entry['title'] . "</option>\n";
            $hasWriteable = true;
        }
    }

    if ($hasWriteable) {
        require ODIN_TEMPLATES . '/data/import.inc';
    }
    require ODIN_TEMPLATES . '/data/export.inc';
} else {
    switch ($importID) {

    case IMPORT_MAPPED:
        if (count($dates) > 0) {
            $actionUrl = Horde::applicationUrl('data.php');
            require $registry->get('templates', 'horde') . '/data/datemap.inc';
        }
        break;

    }
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
