<?php
/**
 * LogParser:: defines an API for log parsing classes.
 *
 * $Horde: odin/lib/LogParser.php,v 1.6 2004/05/23 17:09:24 jan Exp $
 *
 * @author Chuck Hagenbuch <chuck@horde.org>
 * @version $Revision: 1.6 $
 * @package Odin
 */
class LogParser {

    /**
     * Attempts to return a concrete LogParser instance based on $type.
     *
     * @param $type     The type of LogParser subclass to return.  The
     *                  code is dynamically included.
     * @param $params   Hash containing additional paramters to be passed to
     *                  the subclass's constructor.
     *
     * @return          The newly created concrete LogParser instance, or
     *                  false on an error.
     */
    function &factory($type, $params = array())
    {
        $type = basename($type);
        include_once dirname(__FILE__) . '/LogParser/' . $type . '.php';
        $class = 'LogParser_' . $type;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            Horde::fatal(PEAR::raiseError(sprintf(_("Unable to load the definition of %s."), $class)), __FILE__, __LINE__);
        }
    }

    /**
     * Attempts to return a reference to a concrete LogParser instance
     * based on $type. It will only create a new instance if no
     * LogParser instance with the same parameters currently exists.
     *
     * This method must be invoked as: $var = &LogParser::singleton()
     *
     * @param $type     The type of concrete LogParser subclass to return.
     *                  This is based on the storage driver ($type). The
     *                  code is dynamically included.
     * @param $params   A hash containing additional parameters for the
     *                  subclass.
     *
     * @return          The concrete LogParser reference, or false on an
     *                  error.
     */
    function &singleton($type, $params)
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($type, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &LogParser::factory($type, $params);
        }

        return $instances[$signature];
    }

}
