<?php
/**
 * $Horde: odin/query.php,v 1.23 2004/11/22 03:40:44 chuck Exp $
 *
 * Odin: Copyright 2002-2004-2004 Charles J. Hagenbuch <chuck@horde.org>
 *
 * You should have received a copy of the GNU Public
 * License along with this package; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

@define('ODIN_BASE', dirname(__FILE__));
require_once ODIN_BASE . '/lib/base.php';
require_once ODIN_BASE . '/lib/Source.php';
require ODIN_BASE . '/config/attributes.php';

/* Sort out the sorting values. */
if (Util::getFormData('sortby') !== null) {
    $prefs->setValue('sortby', Util::getFormData('sortby'));
    $prefs->store();
}

if (Util::getFormData('sortdir') !== null) {
    $prefs->setValue('sortdir', Util::getFormData('sortdir'));
    $prefs->store();
}

$title = _("Advanced Search");
require ODIN_TEMPLATES . '/common-header.inc';
require ODIN_TEMPLATES . '/query/javascript.inc';

/* Make sure the search session variable is registered in the session,
   and obtain a global-scope reference to it. */
if (!array_key_exists('odin_search_results', $_SESSION)) {
    $_SESSION['odin_search_results'] = null;
} else {
    if (Util::getFormData('clear')) {
        $_SESSION['odin_search_results'] = null;
    } elseif (!empty($_SESSION['odin_search_results'])) {
        require_once ODIN_BASE . '/lib/List.php';
        require_once ODIN_BASE . '/lib/Object.php';
        $_SESSION['odin_search_results'] = Odin_List::unserialize($_SESSION['odin_search_results']);
    }
}

$advanced = Util::getFormData('advanced', 0);

/* Run search if there is one. */
$source = Util::getFormData('source');
if (!isset($source) && isset($cfgSources) && is_array($cfgSources) && count($cfgSources) > 0) {
    reset($cfgSources);
    $source = key($cfgSources);
}

if ($advanced) {
    /* Build the data source select widget. */
    if (count($cfgSources) > 1) {
        $source_options  = '<tr><td class="light" align="right"><b>' . _("Data Source") . '</b></td>';
        $source_options .= '<td align="left"><select name="source" onchange="dirsearch.submit()">';
        foreach ($cfgSources as $key => $entry) {
            $selection = (isset($source) && $source == $key) ? ' selected="selected"' : '';
            $source_options .= "<option value=\"$key\"$selection>";
            $source_options .= $entry['title'] . "</option>\n";
        }
        $source_options .= '</select></td></tr>';
    } else {
        $source_options = '<input type="hidden" name="source" value="';
        reset($cfgSources);
        $source_options .= key($cfgSources);
        $source_options .= '" />';
    }

    $driver = &Odin_Source::singleton($source, $cfgSources[$source]);
    if (PEAR::isError($driver)) {
        Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
        $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
        $map = array();
    } else {
        $map = $driver->getCriteria();

        require ODIN_BASE . '/config/attributes.php';
        $criteria = array();
        foreach ($map as $key => $value) {
            switch ($attributes[$key]['type']) {
            case '__key':
                break;

            case 'yyyymmdd':
                $date = Util::getFormData($key);

                if (!empty($date['start']['year']) &&
                    !empty($date['start']['month']) &&
                    !empty($date['start']['day'])) {
                    $criteria[$key]['>='] = sprintf('%04d%02d%02d',
                                                    $date['start']['year'],
                                                    $date['start']['month'],
                                                    $date['start']['day']);
                }

                if (!empty($date['end']['year']) &&
                    !empty($date['end']['month']) &&
                    !empty($date['end']['day'])) {
                    $criteria[$key]['<='] = sprintf('%04d%02d%02d',
                                                    $date['end']['year'],
                                                    $date['end']['month'],
                                                    $date['end']['day']);
                }
                break;

            case 'hh:mm:ss':
                $time = Util::getFormData($key);

                if (!empty($time['start']['hour']) &&
                    !empty($time['start']['min'])) {
                    $criteria[$key]['>='] = sprintf('%02d:%02d:00',
                                                    $time['start']['hour'],
                                                    $time['start']['min']);
                }

                if (!empty($time['end']['hour']) &&
                    !empty($time['end']['min'])) {
                    $criteria[$key]['<='] = sprintf('%02d:%02d:59',
                                                    $time['end']['hour'],
                                                    $time['end']['min']);
                }
                break;

            default:
                $val = Util::getFormData($key);
                if (!empty($val)) {
                    $criteria[$key] = $val;
                }
                break;
            }
        }

        $match = Util::getFormData('match', 1);
        if ($match == '') {
            $match = 1;
        }

        if (count($criteria)) {
            if (is_object($results = $driver->search($criteria, 'number', $match))) {
                if (!is_object($_SESSION['odin_search_results'])) {
                    $_SESSION['odin_search_results'] = &new Odin_List();
                }
                if ($results->count() == 0) {
                    $notification->push(_("Your search did not match any records."));
                }
                $_SESSION['odin_search_results']->merge($results);
            } else {
                $notification->push(sprintf(_("Failed to search the data source: %s"), ''), 'horde.error');
            }
        }
    }
    require ODIN_TEMPLATES . '/menu.inc';
    require ODIN_TEMPLATES . '/advanced/head.inc';

    foreach ($map as $name => $v) {
        require ODIN_TEMPLATES . '/advanced/criteria.inc';
    }
} else {
    $criteria = Util::getFormData('criteria');
    $val = Util::getFormData('val');
    if (isset($criteria) && isset($val)) {
        $driver = &Odin_Source::singleton($source, $cfgSources[$source]);
        if (PEAR::isError($driver)) {
            Horde::logMessage($driver, __FILE__, __LINE__, PEAR_LOG_ERR);
            $notification->push(_("Failed to connect to the specified data source."), 'horde.error');
        } else {
            if (is_object($results = $driver->search(array($criteria => $val)))) {
                if (!is_object($_SESSION['odin_search_results'])) {
                    $_SESSION['odin_search_results'] = &new Odin_List();
                }
                if ($results->count() == 0) {
                    $notification->push(_("Your search did not match any records."));
                }
                $_SESSION['odin_search_results']->merge($results);
            } else {
                $notification->push(sprintf(_("Failed to search the data source: %s"), ''), 'horde.error');
            }
        }
    }
    require ODIN_TEMPLATES . '/menu.inc';
    require ODIN_TEMPLATES . '/query/search.inc';
}

echo '</table></form>';

// Read the columns to display from the preferences
$sources = Odin::string2Columns($GLOBALS['prefs']->getValue('columns'));
$columns = isset($sources[$source]) ? $sources[$source] : array();

$addresses = Util::getFormData('addresses');

/* Determine the name of the column to sort by */
$sortcolumn = ($prefs->getValue('sortby') == 0 ||
               !isset($columns[$prefs->getValue('sortby') - 1])) ?
    'number' : $columns[$prefs->getValue('sortby') - 1];
$numDisplayed = 0;
if (is_object($_SESSION['odin_search_results']) && $_SESSION['odin_search_results']->count() > 0) {
    require_once ODIN_BASE . '/lib/List.php';
    $_SESSION['odin_search_results']->sort($sortcolumn, $prefs->getValue('sortdir'));

    $title =_("Search Results");
    require ODIN_TEMPLATES . '/query/header.inc';

    $listType = 'search';
    require ODIN_TEMPLATES . '/query/column_headers.inc';

    require_once ODIN_BASE . '/lib/ListView.php';
    $display = &new Odin_ListView($_SESSION['odin_search_results'], ODIN_TEMPLATES . '/query/row.inc');
    $numDisplayed = $display->display();
    $_SESSION['odin_search_results'] = $_SESSION['odin_search_results']->serialize();

    require ODIN_TEMPLATES . '/query/column_footers.inc';
} elseif (!$advanced) { ?>
<table width="50%"><tr><td class="light">
  <?php echo _("Please select a field to search by, enter text in the text box, and click the Search button to conduct the search. If you would like to search additional fields (or multiple fields), please click the Advanced Search button instead.") ?>
</td></tr></table>
<?php }

require ODIN_TEMPLATES . '/query/footer.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
