#!/usr/local/bin/php
<?php
// $Horde: odin/scripts/smap_inject.php,v 1.3 2002/10/10 15:07:36 chuck Exp $

/** Table definition:

CREATE TABLE smap_logs (
    log_id BIGINT NOT NULL,
    log_date VARCHAR(8) NOT NULL,
    log_time VARCHAR(8) NOT NULL,
    log_pid INT,
    log_smaphost VARCHAR(100),
    log_rejected VARCHAR(10),
    log_to VARCHAR(255),
    log_from VARCHAR(255),
    log_fromhost VARCHAR (255),
    log_rule VARCHAR(255),
    log_type VARCHAR(40),
    log_pattern TEXT,

    PRIMARY KEY (log_id)
);

*/

require_once 'DB.php';
$db = &DB::connect(array('phptype' => 'mysql',
                         'hostspec' => 'localhost',
                         'username' => 'bucket',
                         'password' => 'bucket',
                         'database' => 'bucket'));

if (!empty($argv[1])) {
    $logfile = trim($argv[2]);
} else {
    $logfile = './messages';
}

if (!@file_exists($logfile)) {
    exit("Logfile '" . $logfile . "' does not exist.\n");
}

$incidents = array();

$fp = @fopen($logfile, 'r');
if (!$fp) {
    exit("Unable to open logfile '" . $logfile . "'.\n");
}

echo "Processing $logfile...\n";
while ($line = fgets($fp, 4096)) {
	trim($line);
	if (!preg_match('/(\w{3}) +(\w{1,2}) (\d{2}:\d{2}:\d{2}) (\w+) (smap\w*)\[(\d+)\]: (.*)/', $line, $matches)) {
        continue;
    }

	list($whole, $month, $day, $timestamp, $host, $daemon, $pid, $msg) = $matches;

	$incidents[$pid]['date'] = sprintf('%04d%02d%02d', date('Y'), monthFromString($month), $day);
    $incidents[$pid]['time'] = $timestamp;
	$incidents[$pid]['smaphost'] = $host;

	if (strstr($msg, 'Rejected')) {
		$incidents[$pid]['rejected'] = 'yes';

        preg_match('/Rule: (.+?) {2}?(\S+) (.+)/', $msg, $matches);

		$incidents[$pid]['rule'] = $matches[1];
		$incidents[$pid]['pattern'] = $matches[3];
		$type = $matches[2];
        $type = str_replace(':', '', $type);
        $type = preg_replace('/([A-Z]+)/', '\l$1', $type);

		$incidents[$pid]['type'] = $type;
	} elseif (substr($msg, 0, 5) == 'host=' &&
              preg_match('/^host=(.*) bytes=.* from=(.*) to=(.*)/', $msg, $matches)) {
        $incidents[$pid]['matches'] = $matches;

		$fromhost = $matches[1];
		$from = $matches[2];
		$to = $matches[3];

		$from = preg_replace('/[<,>]/', '', $from);
		$to = preg_replace('/[<,>]/', '', $to);

		$incidents[$pid]['fromhost'] = $fromhost;
		$incidents[$pid]['from'] = $from;
		$incidents[$pid]['to'] = $to;
    }
}

$i = 0;
foreach ($incidents as $pid => $row) {
    $id = $db->nextId('smap_logs');
    if (DB::isError($id)) {
        var_dump($id);
        finish($i);
    }
    $query = sprintf('INSERT INTO smap_logs (log_id, log_date, log_time,' .
                     ' log_pid, log_smaphost, log_rejected, log_to, log_from,' .
                     ' log_fromhost, log_rule, log_type, log_pattern)' .
                     ' VALUES (%s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)',
                     (int)$id,                                   // row ID
                     $db->quote($row['date']),                   // date (YYYYMMDD)
                     $db->quote($row['time']),                   // time (HH:MM:SS)
                     (int)$pid,                                  // pid
                     $db->quote($row['smaphost']),               // SMAP host
                     isset($row['rejected']) ? "'yes'" : "'no'", // rejected?
                     $db->quote($row['to']),                     // to
                     $db->quote($row['from']),                   // from
                     $db->quote($row['fromhost']),               // fromhost
                     $db->quote($row['rule']),                   // rule
                     $db->quote($row['type']),                   // type
                     $db->quote($row['pattern']));               // pattern

    $result = $db->query($query);
    if (DB::isError($result)) {
        var_dump($result);
        finish($i);
    }

    $i++;
}

finish($i);


function finish($i)
{
    echo "$i rows inserted.\n";
    exit;
}

function monthFromString($month)
{
    switch ($month) {
    case 'Jan': return 1;
    case 'Feb': return 2;
    case 'Mar': return 3;
    case 'Apr': return 4;
    case 'May': return 5;
    case 'Jun': return 6;
    case 'Jul': return 7;
    case 'Aug': return 8;
    case 'Sep': return 9;
    case 'Oct': return 10;
    case 'Nov': return 11;
    case 'Dec': return 12;
    }
}

function log_timestamp($date)
{
    $parts = preg_split('|\s|', $date);
    if (count($parts) !== 3) {
        return -1;
    }

    $hms = explode(':', $parts[2]);

    $month = monthFromString($parts[0]);

    return mktime($hms[0], $hms[1], $hms[2],
                  $month, $parts[1]);
}
